/*
Copyright (c) 2000,2001, Jelle Kok, University of Amsterdam
All rights reserved.

Redistribution and use in source and binary forms, with or without 
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this 
list of conditions and the following disclaimer. 

2. Redistributions in binary form must reproduce the above copyright notice, 
this list of conditions and the following disclaimer in the documentation 
and/or other materials provided with the distribution. 

3. Neither the name of the University of Amsterdam nor the names of its 
contributors may be used to endorse or promote products derived from this 
software without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE 
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
/*! \file Player.h
<pre>
<b>File:</b>          Player.h
<b>Project:</b>       Robocup Soccer Simulation Team: UvA Trilearn 2001
<b>Authors:</b>       Jelle Kok and Remco de Boer
<b>Created:</b>       03/03/2001
<b>Last Revision:</b> 08/05/2001
<b>Contents:</b>      This file contains the declaration for the Player class,
               which is a superclass from BasicPlayer and contains the
               decision procedure to select the skills from the
               BasicPlayer. 
</pre>
*/

#ifndef PLAYER
#define PLAYER

#include "BasicPlayer.h"
#include "Formations.h"     // needed for Formations

void* stdin_callback( void * v );

/*! This class is a superclass from BasicPlayer and contains a more
    sophisticated decision procedure to determine the next action. */
class Player:public BasicPlayer
{
  Formations    *formations;             /*!< all formation information      */
  bool          bContLoop;               /*!< is server is alive             */

  int           m_lastSay;               /*!< last cycle communicated        */

  void          goalieMainLoop            (                                   );
  void          defenderMainLoop          (                                   );
  void          midfielderMainLoop        (                                   );
  void          attackerMainLoop          (                                   );

public:
  Player                                  ( ActHandler     *a,
                                            WorldModel     *wm,
                                            ServerSettings *ss,
                                            PlayerSettings *cs,
                                            Formations     *fs,
                                            char           *strTeamName,
                                            double         dVersion,
                                            int            iReconnect = -1    );

  VecPosition   getStrategicPosition      ( int            iPlayer = -1       );
  
  // main loop and different team decision procedures
  void          mainLoop                  (                                   );
  void          deMeer5                   (                                   );
  void          deMeer5_goalie            (                                   );

  // methods that deal with user input (from keyboard) to sent commands
  void          handleStdin               (                                   );
  void          showStringCommands        ( ostream& out                      );
  bool          executeStringCommand      ( char *str                         );
  void          test_only_update          (                                   );


  // methods associated with communication
  bool          shallISaySomething        (                                   );
  bool          amIAgentToSaySomething    (                                   );
  SoccerCommand sayWorldStatus            (                                   );


} ;

#endif
